REM &1 = REQPRO_DATA
REM &2 = SMALL
REM &3 = MEDIUM
REM &4 = LARGE
REM &5 = REQPRO_INDEX
REM &6 = USR
REM &7 = PWD

/* Oracle - Scripting                                                                                                      */
/* Database: ReqPro                                                                                                        */
/* Creation 09/28/2009                                                                                                     */
/* Version: 1.00.0008                                                                                                      */

/* If the default Login and User schema has been used, then create all                                                     */
/* objects under the ReqPro user.  Otherwise, create them under                                                            */
/* the current Login.                                                                                                      */

/* History:                                                                                                                */
/* DATE         DEVELOPER(S)           VERSION   DESCRIPTION                                                               */
/* 09/28/2009   Fabian Lomeli          7.1.1     New set of tables, procedures and triggers to handle RRC syncronization   */


-- Connect using REQPRO user.
connect &6/&7


/*
-- DROP TABLES
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOG';
	IF row_cnt > 0 THEN
		EXECUTE IMMEDIATE 'DROP TABLE RQTLOG CASCADE CONSTRAINTS';
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGCOLUMNS';
	IF row_cnt > 0 THEN
		EXECUTE IMMEDIATE 'DROP TABLE RQTLOGCOLUMNS CASCADE CONSTRAINTS';
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGMAP';
	IF row_cnt > 0 THEN
		EXECUTE IMMEDIATE 'DROP TABLE RQTLOGMAP CASCADE CONSTRAINTS';
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGOBJECTTYPE';
	IF row_cnt > 0 THEN
		EXECUTE IMMEDIATE 'DROP TABLE RQTLOGOBJECTTYPE CASCADE CONSTRAINTS';
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGOVERFLOW';
	IF row_cnt > 0 THEN
		EXECUTE IMMEDIATE 'DROP TABLE RQTLOGOVERFLOW CASCADE CONSTRAINTS';
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGPROPERTY';
	IF row_cnt > 0 THEN
		EXECUTE IMMEDIATE 'DROP TABLE RQTLOGPROPERTY CASCADE CONSTRAINTS';
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGTRANSTYPE';
	IF row_cnt > 0 THEN
		EXECUTE IMMEDIATE 'DROP TABLE RQTLOGTRANSTYPE CASCADE CONSTRAINTS';
	END IF;
END;
/
*/



/****** Object:  Table [RQTLOGOBJECTTYPE] ******/
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGOBJECTTYPE';
	IF row_cnt = 0 THEN
		-- If does not exist then create the table
		EXECUTE IMMEDIATE '
			CREATE TABLE RQTLOGOBJECTTYPE
			(
				ID NUMBER(10) NOT NULL,
				OBJECTTYPE VARCHAR2(20) NOT NULL,
				CONSTRAINT PK_RQTLOGOBJECTTYPE PRIMARY KEY (ID)
					USING INDEX
					TABLESPACE &5
					STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			)
			TABLESPACE &1
			STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			';
	END IF;
END;
/


/****** Object:  Table [RQTLOGTRANSTYPE] ******/
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGTRANSTYPE';
	IF row_cnt = 0 THEN
		-- If does not exist then create the table
		EXECUTE IMMEDIATE '
			CREATE TABLE RQTLOGTRANSTYPE
			(
				ID NUMBER(10) NOT NULL,
				TRANSTYPE VARCHAR2(20) NOT NULL,
				CONSTRAINT PK_RQTLOGTRANSTYPE PRIMARY KEY (ID)
					USING INDEX
					TABLESPACE &5
					STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			)
			TABLESPACE &1
			STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			';
	END IF;
END;
/


/****** Object:  Table [RQTLOG] ******/
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOG';
	IF row_cnt = 0 THEN
		-- If does not exist then create the table
		EXECUTE IMMEDIATE '
			CREATE TABLE RQTLOG
			(
				ID NUMBER(10) NOT NULL,
				PROJECTGUID VARCHAR2(40) NOT NULL,
				PROJECTMAPID VARCHAR2(40) NULL,
				USERID VARCHAR2(20) NOT NULL,
				TDATE DATE NOT NULL,
				ISDOCUMENT NUMBER(1) NOT NULL,
				RQTTRANSTYPEID NUMBER(10) NOT NULL,
				RQTOBJECTTYPEID NUMBER(10) NOT NULL,
				RQTOBJECTGUID VARCHAR2(40) NOT NULL,
				RQTEXTERNALID VARCHAR2(40) NULL,
				CONSTRAINT PK_RQTLOG PRIMARY KEY (ID)
					USING INDEX
					TABLESPACE &5
					STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			)
			TABLESPACE &1
			STORAGE(INITIAL &4 NEXT &4 PCTINCREASE 0)
			';
	END IF;
END;
/


DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_OBJECTS
		WHERE OBJECT_TYPE = 'SEQUENCE' AND OBJECT_NAME = 'SEQ_RQTLOG_ID';
	IF row_cnt = 0 THEN
		-- If does not exist then create the Sequence
		EXECUTE IMMEDIATE '
			CREATE SEQUENCE SEQ_RQTLOG_ID
				MINVALUE 1
				MAXVALUE 9999999999
				START WITH 1
				INCREMENT BY 1
				NOCACHE
			';
	END IF;
END;
/


CREATE OR REPLACE TRIGGER TRG_RQTLOG_INSERT
	BEFORE INSERT ON RQTLOG
	FOR EACH ROW
BEGIN
	IF :new.ID IS NULL THEN
		SELECT SEQ_RQTLOG_ID.nextval INTO :new.ID FROM DUAL;
	END IF;
END;
/


DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_CONSTRAINTS
		WHERE CONSTRAINT_NAME = 'FK_RQTLOG_RQTLOGOBJECTTYPE';
	IF row_cnt = 0 THEN
		-- If does not exist then create the Constraint
		EXECUTE IMMEDIATE '
			ALTER TABLE RQTLOG
				ADD  ( CONSTRAINT FK_RQTLOG_RQTLOGOBJECTTYPE FOREIGN KEY (RQTOBJECTTYPEID)
				REFERENCES RQTLOGOBJECTTYPE(ID))
			';
	END IF;
END;
/


DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_CONSTRAINTS
		WHERE CONSTRAINT_NAME = 'FK_RQTLOG_RQTLOGTRANSTYPE';
	IF row_cnt = 0 THEN
		-- If does not exist then create the Constraint
		EXECUTE IMMEDIATE '
			ALTER TABLE RQTLOG
				ADD  ( CONSTRAINT FK_RQTLOG_RQTLOGTRANSTYPE FOREIGN KEY (RQTTRANSTYPEID)
				REFERENCES RQTLOGTRANSTYPE(ID)) 
			';
	END IF;
END;
/


/****** Object:  Table [RQTLOGCOLUMNS] ******/
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGCOLUMNS';
	IF row_cnt = 0 THEN
		-- If does not exist then create the table
		EXECUTE IMMEDIATE '
			CREATE TABLE RQTLOGCOLUMNS
			(
				ID NUMBER(10) NOT NULL,
				RQTLOGID NUMBER(10) NOT NULL,
				TABLENAME VARCHAR2(150) NOT NULL,
				TID VARCHAR2(40) NOT NULL,
				COLUMNNAME VARCHAR2(200) NOT NULL,
				COLUMNVALUE VARCHAR(4000) NULL,
				ISOVERFLOW NUMBER(1) NOT NULL,
				CONSTRAINT PK_RQTLOGCOLUMNS PRIMARY KEY (ID)
					USING INDEX
					TABLESPACE &5
					STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			)
			TABLESPACE &1
			STORAGE(INITIAL &4 NEXT &4 PCTINCREASE 0)
			';
	END IF;
END;
/


DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_OBJECTS
		WHERE OBJECT_TYPE = 'SEQUENCE' AND OBJECT_NAME = 'SEQ_RQTLOGCOLUMNS_ID';
	IF row_cnt = 0 THEN
		-- If does not exist then create the Sequence
		EXECUTE IMMEDIATE '
			CREATE SEQUENCE SEQ_RQTLOGCOLUMNS_ID
				MINVALUE 1
				MAXVALUE 9999999999
				START WITH 1
				INCREMENT BY 1
				NOCACHE
			';
	END IF;
END;
/


CREATE OR REPLACE TRIGGER TRG_RQTLOGCOLUMNS_INSERT
   BEFORE INSERT ON RQTLOGCOLUMNS
   FOR EACH ROW
BEGIN
   IF :new.ID IS NULL THEN
      SELECT SEQ_RQTLOGCOLUMNS_ID.nextval INTO :new.ID FROM DUAL;
   END IF;
END;
/


DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_CONSTRAINTS
		WHERE CONSTRAINT_NAME = 'FK_RQTLOGCOLUMNS_RQTLOG';
	IF row_cnt = 0 THEN
		-- If does not exist then create the Constraint
		EXECUTE IMMEDIATE '
			ALTER TABLE RQTLOGCOLUMNS
				ADD  ( CONSTRAINT FK_RQTLOGCOLUMNS_RQTLOG FOREIGN KEY (RQTLOGID)
				REFERENCES RQTLOG(ID) ON DELETE CASCADE)
			';
	END IF;
END;
/


/****** Object:  Table [RQTLOGOVERFLOW] ******/
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGOVERFLOW';
	IF row_cnt = 0 THEN
		-- If does not exist then create the table
		EXECUTE IMMEDIATE '
			CREATE TABLE RQTLOGOVERFLOW
			(
				RQTLOGCOLUMNSID NUMBER(10) NOT NULL,
				OVERFLOWTEXT LONG NULL
			)
			TABLESPACE &1
			STORAGE(INITIAL &4 NEXT &4 PCTINCREASE 0)
			';
	END IF;
END;
/


DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_CONSTRAINTS
		WHERE CONSTRAINT_NAME = 'FK_RQTLOGOVERFLOW_RQTLOGCOLUMN';
	IF row_cnt = 0 THEN
		-- If does not exist then create the Constraint
		EXECUTE IMMEDIATE '
			ALTER TABLE RQTLOGOVERFLOW
			    ADD  ( CONSTRAINT FK_RQTLOGOVERFLOW_RQTLOGCOLUMN FOREIGN KEY (RQTLOGCOLUMNSID)
				REFERENCES RQTLOGCOLUMNS(ID) ON DELETE CASCADE)
			';
	END IF;
END;
/


/****** Object:  Table [RQTLOGMAP] ******/
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGMAP';
	IF row_cnt = 0 THEN
		-- If does not exist then create the table
		EXECUTE IMMEDIATE '
			CREATE TABLE RQTLOGMAP
			(
				ID NUMBER(10) NOT NULL,
				OBJECTTYPE VARCHAR2(30) NOT NULL,
				OBJECTGUID VARCHAR2(200) NOT NULL,
				EXTERNALID VARCHAR2(40) NOT NULL,
				CONSTRAINT PK_RQTLOGMAP PRIMARY KEY (ID)
					USING INDEX
					TABLESPACE &5
					STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0),
				CONSTRAINT UK_RQTLOGMAP UNIQUE (OBJECTTYPE, OBJECTGUID)
					USING INDEX
					TABLESPACE &5
					STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			)
			TABLESPACE &1
			STORAGE(INITIAL &3 NEXT &3 PCTINCREASE 0)
			';
	END IF;
END;
/
	

DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_OBJECTS
		WHERE OBJECT_TYPE = 'SEQUENCE' AND OBJECT_NAME = 'SEQ_RQTLOGMAP_ID';
	IF row_cnt = 0 THEN
		-- If does not exist then create the Sequence
		EXECUTE IMMEDIATE '
			CREATE SEQUENCE SEQ_RQTLOGMAP_ID
				MINVALUE 1
				MAXVALUE 9999999999
				START WITH 1
				INCREMENT BY 1
				NOCACHE
			';
	END IF;
END;
/


CREATE OR REPLACE TRIGGER TRG_RQTLOGMAP_INSERT
	BEFORE INSERT ON RQTLOGMAP
	FOR EACH ROW
BEGIN
	IF :new.ID IS NULL THEN
		SELECT SEQ_RQTLOGMAP_ID.nextval INTO :new.ID FROM DUAL;
	END IF;
END;
/


/****** Object:  Table [RQTLOGPROPERTY] ******/
DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM ALL_TABLES
		WHERE TABLE_NAME = 'RQTLOGPROPERTY';
	IF row_cnt = 0 THEN
		-- If does not exist then create the table
		EXECUTE IMMEDIATE '
			CREATE TABLE RQTLOGPROPERTY
			(
				PROJECTGUID VARCHAR2(40) NOT NULL,
				KEYNAME VARCHAR2(100) NOT NULL,
				VALUE VARCHAR2(200) NOT NULL,
				CONSTRAINT PK_RQTLOGPROPERTY PRIMARY KEY (PROJECTGUID, KEYNAME)
					USING INDEX
					TABLESPACE &5
					STORAGE(INITIAL &2 NEXT &2 PCTINCREASE 0)
			)
			TABLESPACE &1
			STORAGE(INITIAL &3 NEXT &3 PCTINCREASE 0)
			';
	END IF;
END;
/


DECLARE
	row_cnt NUMBER;
BEGIN
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGTRANSTYPE WHERE ID = 1;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGTRANSTYPE VALUES (1, 'insert');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGTRANSTYPE WHERE ID = 2;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGTRANSTYPE VALUES (2, 'update');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGTRANSTYPE WHERE ID = 3;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGTRANSTYPE VALUES (3, 'delete');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGOBJECTTYPE WHERE ID = 1;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGOBJECTTYPE VALUES (1, 'project');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGOBJECTTYPE WHERE ID = 2;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGOBJECTTYPE VALUES (2, 'requirementtype');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGOBJECTTYPE WHERE ID = 3;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGOBJECTTYPE VALUES (3, 'attribute');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGOBJECTTYPE WHERE ID = 4;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGOBJECTTYPE VALUES (4, 'package');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGOBJECTTYPE WHERE ID = 5;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGOBJECTTYPE VALUES (5, 'document');
	END IF;
	
	SELECT COUNT(*) INTO row_cnt FROM RQTLOGOBJECTTYPE WHERE ID = 6;
	IF row_cnt = 0 THEN
		INSERT INTO RQTLOGOBJECTTYPE VALUES (6, 'requirement');
	END IF;
END;
/


/****** Object:  StoredProcedure [RQINSERT_RQTLOG] ******/
CREATE OR REPLACE PROCEDURE RQINSERT_RQTLOG
(
   V_PROJECTGUID VARCHAR2,
   V_PROJECTMAPID VARCHAR2,
   V_USERID VARCHAR2,
   V_TDATE DATE,
   V_ISDOCUMENT NUMBER,
   V_RQTTRANSTYPEID NUMBER,
   V_RQTOBJECTTYPEID NUMBER,
   V_RQTOBJECTGUID VARCHAR2,
   V_RQTEXTERNALID VARCHAR2,
   V_LOGSID OUT NUMBER
)
IS
BEGIN   
   -- insert column values into RQTLOG table
   INSERT INTO RQTLOG (PROJECTGUID, PROJECTMAPID, USERID, TDATE, ISDOCUMENT, RQTTRANSTYPEID, RQTOBJECTTYPEID, RQTOBJECTGUID, RQTEXTERNALID)
   VALUES (V_PROJECTGUID, V_PROJECTMAPID, V_USERID, V_TDATE, V_ISDOCUMENT, V_RQTTRANSTYPEID, V_RQTOBJECTTYPEID, V_RQTOBJECTGUID, V_RQTEXTERNALID);
   
   -- Get the value used as ID to the just inserted record
   SELECT SEQ_RQTLOG_ID.CURRVAL INTO V_LOGSID FROM DUAL;
END;
/


/****** Object:  StoredProcedure [RQINSERT_RQTLOGCOLUMNS] ******/
CREATE OR REPLACE PROCEDURE RQINSERT_RQTLOGCOLUMNS
(  
   V_RQTLOGID NUMBER,
   V_TABLENAME VARCHAR2,
   V_TID VARCHAR2,
   V_COLUMNNAME VARCHAR2,
   V_COLUMNVALUE VARCHAR2,
   V_ISOVERFLOW NUMBER,
   V_LOGSCOLUMNID OUT NUMBER
)
IS
BEGIN
   -- insert column values into RQTLOGCOLUMNS table
   INSERT INTO RQTLOGCOLUMNS (RQTLOGID, TABLENAME, TID, COLUMNNAME, COLUMNVALUE, ISOVERFLOW)
   VALUES (V_RQTLOGID, V_TABLENAME, V_TID, V_COLUMNNAME, V_COLUMNVALUE, V_ISOVERFLOW);
   
   -- Get the value used as ID to the just inserted record
   SELECT SEQ_RQTLOGCOLUMNS_ID.CURRVAL INTO V_LOGSCOLUMNID FROM DUAL;
END;
/


/****** Object:  StoredProcedure [RQINSERT_RQTLOGOVERFLOW] ******/
CREATE OR REPLACE PROCEDURE RQINSERT_RQTLOGOVERFLOW
(
   V_RQTLOGCOLUMNSID NUMBER,
   V_OVERFLOWTEXT LONG
)
IS
BEGIN
   -- insert column values into RQTLOGOVERFLOW table
   INSERT INTO RQTLOGOVERFLOW (RQTLOGCOLUMNSID, OVERFLOWTEXT)
   VALUES (V_RQTLOGCOLUMNSID, V_OVERFLOWTEXT);
END;
/


CREATE OR REPLACE PROCEDURE RQINSERT_RQTLOGOVERFLOW_LONG
(
	v_query varchar2,
	v_int_target NUMBER
)
IS
    v_cursor integer default dbms_sql.open_cursor;
    v_n number;
    v_long_val varchar2(32760);
    v_long_len number;
    v_buflen number := 32760;
    v_curpos number;
    v_longtext long;
    v_intvalue number(10);
    v_inserted number(1);
    v_exist_data number(1);
    v_column_int number;
    v_column_long number;
BEGIN
	/***** This block reads from a LONG and insert into a LONG field *****/
	dbms_sql.parse( v_cursor, v_query, dbms_sql.native);
	
	IF v_int_target IS NULL THEN
		-- Set the columns position of the statement being processed.
		-- They are used when binding and retrieving column values
		v_column_int := 1;
		v_column_long := 2;
		
		-- Define the int value since it will be retrieved from the query
		dbms_sql.define_column(v_cursor, v_column_int, v_intvalue);
	ELSE
		-- Set the columns position of the statement being processed.
		-- They are used when binding and retrieving column values
		v_column_int := 1;  -- It is not needed in this case since the int value is passed as param
		v_column_long := 1;
		
		-- Use the value passed
		v_intvalue := v_int_target;
	END IF;
	
	dbms_sql.define_column_long(v_cursor, v_column_long);
	v_n := dbms_sql.execute(v_cursor);
	LOOP
		IF dbms_sql.fetch_rows(v_cursor) = 0 THEN
			-- Break the loop when no more rows
			exit;
		ELSE
			-- Initialize variables before each row processing
			v_curpos := 0;
			v_longtext := '';
			v_inserted := 0;
			v_exist_data := 0;
			
			IF v_int_target IS NULL THEN
				-- Get the r1.ID value
				dbms_sql.column_value(v_cursor, v_column_int, v_intvalue);
			END IF;
			
			-- Loop to get the Long field
			BEGIN
				LOOP
					-- Get a chunk of Long data
					dbms_sql.column_value_long(v_cursor, v_column_long, v_buflen, 
					                           v_curpos , v_long_val,
					                           v_long_len );
					-- Position the next offset
					v_curpos := v_curpos + v_long_len;
					-- Set the flag for data existance to be inserted
					v_exist_data := 1;
					-- Append to the long field
					v_longtext := v_longtext || v_long_val;
					exit when v_long_len = 0;
				END LOOP;
				INSERT INTO RQTLOGOVERFLOW (RQTLOGCOLUMNSID, OVERFLOWTEXT) 
					VALUES (v_intvalue, v_longtext);
				v_inserted := 1;
			EXCEPTION
				WHEN OTHERS THEN
					IF (v_exist_data = 1 AND v_inserted = 0) THEN
						INSERT INTO RQTLOGOVERFLOW (RQTLOGCOLUMNSID, OVERFLOWTEXT) 
							VALUES (v_intvalue, v_longtext);							
					END IF;
		
					IF dbms_sql.is_open(v_cursor) THEN
						dbms_sql.close_cursor(v_cursor);
					END IF;
			END;
		END IF;
	END LOOP;
EXCEPTION
	WHEN OTHERS THEN			
		IF dbms_sql.is_open(v_cursor) THEN
			dbms_sql.close_cursor(v_cursor);
		END IF;
END;
/


/****** Object:  StoredProcedure [RQINS_TLOGCOLUMNS_FIELDVALUES] ******/
CREATE OR REPLACE PROCEDURE RQINS_TLOGCOLUMNS_FIELDVALUES
(
   V_RQTLOGID NUMBER,
   V_REQUIREMENTID NUMBER
)
IS
    V_QUERY VARCHAR2(500);
BEGIN
   -- insert column values into RQTLOGCOLUMNS table
   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r2.RQGUID,'REQUIREMENTID',r1.REQUIREMENTID,0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQUSERDEFINEDFIELDS r2 ON r1.FIELDID=r2.ID
   WHERE r1.REQUIREMENTID = V_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'REQUIREMENTGUID',r2.RQGUID,0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r1.FIELDID=r3.ID
   WHERE r1.REQUIREMENTID = V_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r4.RQGUID,'REQUIREMENTMAPID',EXTERNALID,0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQTLOGMAP r3 ON r2.RQGUID=r3.OBJECTGUID
   INNER JOIN RQUSERDEFINEDFIELDS r4 ON r1.FIELDID=r4.ID
   WHERE r1.REQUIREMENTID = V_REQUIREMENTID AND r3.OBJECTTYPE='RQREQUIREMENTS';

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'FIELDVALUE',r4.FIELDVALUE,nvl(r4.OVERFLOWIND,0)
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   LEFT OUTER JOIN RQUSERDEFINEDFIELDVALUES r4 ON r3.ID=r4.FIELDID AND r1.ID=r4.REQUIREMENTID
   WHERE r1.ID=V_REQUIREMENTID AND r3.DATATYPE<>0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r2.RQGUID,'OVERFLOWIND',r1.OVERFLOWIND,0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQUSERDEFINEDFIELDS r2 ON r1.FIELDID=r2.ID
   WHERE r1.REQUIREMENTID = V_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r2.RQGUID,'NUMERICVALUE',r1.NUMERICVALUE,0
   FROM RQUSERDEFINEDFIELDVALUES r1
   INNER JOIN RQUSERDEFINEDFIELDS r2 ON r1.FIELDID=r2.ID
   WHERE r1.REQUIREMENTID = V_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'FIELDGUID',r3.RQGUID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   WHERE r1.ID=V_REQUIREMENTID AND r3.DATATYPE<>0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDFIELDVALUES',r3.RQGUID,'FIELDMAPID',r4.EXTERNALID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   INNER JOIN RQTLOGMAP r4 ON r3.RQGUID=r4.OBJECTGUID
   WHERE r1.ID = V_REQUIREMENTID 
   AND r3.DATATYPE<>0 
   AND r4.OBJECTTYPE = 'RQUSERDEFINEDFIELDS';

   --insert columns values into RQTLOGOVERFOW table
	V_QUERY := 'SELECT r1.ID, r2.OVERFLOWFIELDVALUE
				FROM RQTLOGCOLUMNS r1
				INNER JOIN RQOVERFLOWVALUES r2 ON r1.TID=r2.OVERFLOWFIELDID
				WHERE r1.RQTLOGID=' || V_RQTLOGID ||'
				AND r1.TABLENAME=''RQUSERDEFINEDFIELDVALUES''
				AND r1.COLUMNNAME=''FIELDVALUE''
				AND r1.ISOVERFLOW=1
				AND r2.OVERFLOWREQUIREMENTID=' || V_REQUIREMENTID;
	RQINSERT_RQTLOGOVERFLOW_LONG(V_QUERY, NULL);
END;
/


/****** Object:  StoredProcedure [RQINS_TLOGCOLUMNS_LISTVALUES] ******/
CREATE OR REPLACE PROCEDURE RQINS_TLOGCOLUMNS_LISTVALUES
(
   V_RQTLOGID NUMBER,
   V_REQUIREMENTID NUMBER
)
IS
BEGIN
   -- insert column values into RQTLOGCOLUMNS table
   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r4.RQGUID,'REQUIREMENTGUID',r2.RQGUID,0
   FROM RQUSERDEFINEDLISTVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQUSERDEFINEDLISTITEMS r3 ON r1.LISTITEMID=r3.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r4 ON r1.FIELDID=r4.ID
   WHERE r1.REQUIREMENTID=V_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r5.RQGUID,'FIELDGUID',r5.RQGUID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   LEFT OUTER JOIN RQUSERDEFINEDLISTVALUES r3 ON r2.USERDEFINEDFIELDID=r3.FIELDID AND r1.ID=r3.REQUIREMENTID
   FULL OUTER JOIN RQUSERDEFINEDLISTITEMS r4 ON r3.LISTITEMID=r4.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r5 ON r2.USERDEFINEDFIELDID=r5.ID
   WHERE r1.ID=V_REQUIREMENTID AND r5.DATATYPE=0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r5.RQGUID,'ITEMGUID',r4.RQGUID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   LEFT OUTER JOIN RQUSERDEFINEDLISTVALUES r3 ON r2.USERDEFINEDFIELDID=r3.FIELDID AND r1.ID=r3.REQUIREMENTID
   FULL OUTER JOIN RQUSERDEFINEDLISTITEMS r4 ON r3.LISTITEMID=r4.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r5 ON r2.USERDEFINEDFIELDID=r5.ID
   WHERE r1.ID=V_REQUIREMENTID AND r5.DATATYPE=0;

   -- insert external MAP ID values
   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r5.RQGUID,'REQUIREMENTMAPID',r3.EXTERNALID,0
   FROM RQUSERDEFINEDLISTVALUES r1
   INNER JOIN RQREQUIREMENTS r2 ON r1.REQUIREMENTID=r2.ID
   INNER JOIN RQTLOGMAP r3 ON r2.RQGUID=r3.OBJECTGUID
   INNER JOIN RQUSERDEFINEDLISTITEMS r4 ON r1.LISTITEMID=r4.ITEMID
   INNER JOIN RQUSERDEFINEDFIELDS r5 ON r1.FIELDID=r5.ID
   WHERE r1.REQUIREMENTID=V_REQUIREMENTID;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r3.RQGUID,'FIELDMAPID',r4.EXTERNALID,0
   FROM RQREQUIREMENTS r1
   INNER JOIN RQREQUIREMENTTYPEFIELDS r2 ON r1.REQUIREMENTTYPEID=r2.REQUIREMENTTYPEID
   INNER JOIN RQUSERDEFINEDFIELDS r3 ON r2.USERDEFINEDFIELDID=r3.ID
   INNER JOIN RQTLOGMAP r4 ON r3.RQGUID=r4.OBJECTGUID
   WHERE r1.ID=V_REQUIREMENTID AND r3.DATATYPE=0;

   INSERT INTO RQTLOGCOLUMNS (RQTLOGID,TABLENAME,TID,COLUMNNAME,COLUMNVALUE,ISOVERFLOW)
   SELECT V_RQTLOGID,'RQUSERDEFINEDLISTVALUES',r4.RQGUID,'ITEMMAPID',r3.EXTERNALID,0
   FROM RQUSERDEFINEDLISTVALUES r1
   INNER JOIN RQUSERDEFINEDLISTITEMS r2 ON r1.LISTITEMID=r2.ITEMID
   INNER JOIN RQTLOGMAP r3 ON r2.RQGUID=r3.OBJECTGUID
   INNER JOIN RQUSERDEFINEDFIELDS r4 ON r1.FIELDID=r4.ID
   WHERE r1.REQUIREMENTID=V_REQUIREMENTID;
END;
/


/****** Object:  StoredProcedure [RQINS_TLOGCOLUMNS_REQ] ******/
CREATE OR REPLACE PROCEDURE RQINS_TLOGCOLUMNS_REQ
(
	V_TLOGID NUMBER,
	V_REQUIREMENTID NUMBER
)
IS
	--Declarations of RQRequirement columns values
    V_RQREQUIREMENTID NUMBER(10);
    V_REQUIREMENTTYPEID NUMBER(10);
    V_TAG NUMBER(10);
    V_LASTCHILDTAG NUMBER(10);
    V_BOOKMARKNAME VARCHAR2(64);
    V_BOOKMARKDOCUMENTID NUMBER(10);
    V_DOCUMENTPOSITION NUMBER(6);
    V_LONGTAGNUMBER VARCHAR2(255);
    V_REQUIREMENTPREFIX VARCHAR2(64);
    V_REQUIREMENTTEXT VARCHAR2(4000);
    V_REQOVERFLOWIND NUMBER(1);
    V_DELETED NUMBER(1);
    V_RQGUID VARCHAR2(40);
    V_REQFLAGS NUMBER(10);
    V_LASTDOCTEXT NUMBER(10);
    V_REQUIREMENTNAME VARCHAR(255);
    V_SUSPECTDATETIME DATE;

	--Local variable declarations
    V_LOGSCOLUMNTID NUMBER(10);
    V_RQDOCGUID VARCHAR2(40);
    V_RQREQTYPEGUID VARCHAR2(40);
    V_RQREQTYPEMAPID VARCHAR2(40);
    V_EXTERNALID VARCHAR2(40);
    V_QUERY VARCHAR2(500);
BEGIN
	--Populate values from the RQREQUIREMENTS table
	SELECT ID,
		nvl(REQUIREMENTTYPEID,''),
		nvl(TAG,-1),
		nvl(LASTCHILDTAG,-1),
		nvl(BOOKMARKNAME,''),
		nvl(BOOKMARKDOCUMENTID,-1),
		nvl(DOCUMENTPOSITION,-1),
		nvl(LONGTAGNUMBER,''),
		nvl(REQUIREMENTPREFIX,''),
		nvl(REQUIREMENTTEXT,''),
		nvl(OVERFLOWIND,0),
		nvl(DELETED,-1),
		nvl(RQGUID,''),
		nvl(FLAGS,-1),
		nvl(LASTDOCTEXT,-1),
		nvl(REQUIREMENTNAME,''),
		nvl(SUSPECTDATETIME,'')
	INTO V_RQREQUIREMENTID,
		V_REQUIREMENTTYPEID,
		V_TAG,
		V_LASTCHILDTAG,
		V_BOOKMARKNAME,
		V_BOOKMARKDOCUMENTID,
		V_DOCUMENTPOSITION,
		V_LONGTAGNUMBER,
		V_REQUIREMENTPREFIX,
		V_REQUIREMENTTEXT,
		V_REQOVERFLOWIND,
		V_DELETED,
		V_RQGUID,
		V_REQFLAGS,
		V_LASTDOCTEXT,
		V_REQUIREMENTNAME,
		V_SUSPECTDATETIME		
	FROM RQREQUIREMENTS
	WHERE ID = V_REQUIREMENTID;
	
	IF V_RQREQUIREMENTID IS NOT NULL THEN
		BEGIN
			-- get guids from id
			SELECT RQGUID INTO V_RQREQTYPEGUID FROM RQREQUIREMENTTYPES WHERE ID=V_REQUIREMENTTYPEID;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_RQREQTYPEGUID := NULL;
		END;
		
		BEGIN		
			SELECT RQGUID INTO V_RQDOCGUID FROM RQDOCUMENTS WHERE ID=V_BOOKMARKDOCUMENTID;
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_RQDOCGUID := NULL;
		END;
		
		BEGIN
			-- get external map ids from RQTLOGMAP table
			SELECT EXTERNALID INTO V_RQREQTYPEMAPID FROM RQTLOGMAP WHERE OBJECTGUID=V_RQREQTYPEGUID AND OBJECTTYPE='RQREQUIREMENTTYPES';
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_RQREQTYPEMAPID := NULL;
		END;
		
		BEGIN
			SELECT EXTERNALID INTO V_EXTERNALID FROM RQTLOGMAP WHERE OBJECTGUID=V_RQGUID AND OBJECTTYPE='RQREQUIREMENTS';
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_EXTERNALID := NULL;
		END;
		
		--insert column values into RQTLOGCOLUMNS table
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'REQUIREMENTID', V_RQREQUIREMENTID, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'REQUIREMENTTYPEGUID', V_RQREQTYPEGUID, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'REQUIREMENTTYPEMAPID', V_RQREQTYPEMAPID, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'TAG', V_TAG, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'LASTCHILDTAG', V_LASTCHILDTAG, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'BOOKMARKNAME', V_BOOKMARKNAME, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'BOOKMARKDOCUMENTID', V_BOOKMARKDOCUMENTID, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'BOOKMARKDOCUMENTGUID', V_RQDOCGUID, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'DOCUMENTPOSITION', V_DOCUMENTPOSITION, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'LONGTAGNUMBER', V_LONGTAGNUMBER, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'REQUIREMENTPREFIX', V_REQUIREMENTPREFIX, 0, V_LOGSCOLUMNTID);
		
		--Check whether an overflow exists. If exists populate the RQTLOGOVERFLOW table for the REQUIREMENTTEXT column entry
		IF V_REQOVERFLOWIND = 1 THEN
			RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'REQUIREMENTTEXT',  V_REQUIREMENTTEXT,  1, V_LOGSCOLUMNTID);
			
			V_QUERY := 'SELECT OVERFLOWTEXT FROM RQOVERFLOWTEXT WHERE OVERFLOWID = ' || V_RQREQUIREMENTID;
			RQINSERT_RQTLOGOVERFLOW_LONG(V_QUERY, V_LOGSCOLUMNTID);
		ELSE
			RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'REQUIREMENTTEXT', V_REQUIREMENTTEXT, 0, V_LOGSCOLUMNTID);
		END IF;
		
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'OVERFLOWIND', V_REQOVERFLOWIND, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'DELETED', V_DELETED, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'RQGUID', V_RQGUID, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'MAPID', V_EXTERNALID, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'FLAGS', V_REQFLAGS, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'LASTDOCTEXT', V_LASTDOCTEXT, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'REQUIREMENTNAME', V_REQUIREMENTNAME, 0, V_LOGSCOLUMNTID);
		RQINSERT_RQTLOGCOLUMNS (V_TLOGID, 'RQREQUIREMENTS', V_RQGUID, 'SUSPECTDATETIME', V_SUSPECTDATETIME, 0, V_LOGSCOLUMNTID);
	END IF;
END;
/


/****** Object:  StoredProcedure [RQDELETE_RQTLOGMAP_ENTRY] ******/
CREATE OR REPLACE PROCEDURE RQDELETE_RQTLOGMAP_ENTRY
(
	V_OBJECTTYPE VARCHAR2,
	V_OBJECTGUID VARCHAR2
)
IS
	V_RQREQUIREMENTID NUMBER(10);
BEGIN
   -- Delete the RQTLOGMAP entry for this requirement.
   DELETE FROM RQTLOGMAP WHERE OBJECTTYPE=V_OBJECTTYPE and OBJECTGUID=V_OBJECTGUID;

   IF upper(rtrim(V_OBJECTTYPE))='RQREQUIREMENTS' THEN
      -- Delete the RQRESOURCES entry for this requirement.
      SELECT ID INTO V_RQREQUIREMENTID FROM RQREQUIREMENTS WHERE RQGUID = V_OBJECTGUID;
      DELETE FROM RQRESOURCES WHERE FLAGS=2 AND REQUIREMENTID=V_RQREQUIREMENTID;
   END IF;

END;
/


/****** Object:  StoredProcedure [RQINSERT_LOG_REQUIREMENT] ******/
CREATE OR REPLACE PROCEDURE RQINSERT_LOG_REQUIREMENT
(
	V_PROJECTGUID VARCHAR2,
	V_REQUIREMENTGUID VARCHAR2,
	V_TRANSTYPE NUMBER
)
IS
	--Declarations of RQRequirement columns values
	V_RQREQUIREMENTID NUMBER(10);
	V_VERSIONUSERID NUMBER(10);
	V_VERSIONDATETIME DATE;
	V_BOOKMARKDOCUMENTID NUMBER(10);

	--Local variable declarations
	V_LOGSTID NUMBER(10);
	V_ISDOC NUMBER(1);
	V_DOCVERNUMBER VARCHAR2(50);
	V_RQPROJECTMAPID VARCHAR2(40);
	V_RQUSERID VARCHAR2(20);
	V_RQDOCGUID VARCHAR2(40);
--	V_RQDOCMAPID VARCHAR2(40);
	V_EXTERNALID VARCHAR2(40);
BEGIN
	--Populate values from the RQREQUIREMENTS table
	SELECT ID,
		nvl(BOOKMARKDOCUMENTID,-1),
		nvl(VERSIONUSERID,-1),
		nvl(VERSIONDATETIME,SYSDATE)
	INTO V_RQREQUIREMENTID,
		V_BOOKMARKDOCUMENTID,
		V_VERSIONUSERID,
		V_VERSIONDATETIME
	FROM RQREQUIREMENTS WHERE RQGUID = V_REQUIREMENTGUID;

	IF V_RQREQUIREMENTID IS NOT NULL THEN
		--Set default values
		V_ISDOC := 0;
		IF V_BOOKMARKDOCUMENTID > 0 THEN
			V_ISDOC := 1;
		END IF;

		-- get user guid and login name from id
		SELECT nvl(LOGINNAME,'UNKNOWN') INTO V_RQUSERID FROM RQUSERS WHERE ID=V_VERSIONUSERID;
		
		-- get external map ids from RQTLOGMAP table
		BEGIN
			SELECT EXTERNALID INTO V_RQPROJECTMAPID FROM RQTLOGMAP WHERE OBJECTGUID=V_PROJECTGUID AND OBJECTTYPE='RQPROJECTS';
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_RQPROJECTMAPID := NULL;
		END;
		BEGIN
			SELECT EXTERNALID INTO V_EXTERNALID FROM RQTLOGMAP WHERE OBJECTGUID=V_REQUIREMENTGUID AND OBJECTTYPE='RQREQUIREMENTS';
		EXCEPTION
			WHEN NO_DATA_FOUND THEN
				V_EXTERNALID := NULL;
		END;

		-- insert column values into RQTLOG table
		RQINSERT_RQTLOG (V_PROJECTGUID, V_RQPROJECTMAPID, V_RQUSERID, V_VERSIONDATETIME, V_ISDOC,
			   V_TRANSTYPE, 6, V_REQUIREMENTGUID, V_EXTERNALID, V_LOGSTID);

		IF V_TRANSTYPE <> 3 THEN
			--insert for RQREQUIREMENTS
			--insert column values into RQTLOGCOLUMNS table
			RQINS_TLOGCOLUMNS_REQ (V_LOGSTID, V_RQREQUIREMENTID);

			--insert for RQUSERDEFINEDFIELDVALUES
			--insert column values into RQTLOGCOLUMNS table
			RQINS_TLOGCOLUMNS_FIELDVALUES (V_LOGSTID, V_RQREQUIREMENTID);

			--insert for RQUSERDEFINEDLISTITEMS
			--insert column values into RQTLOGCOLUMNS table
			RQINS_TLOGCOLUMNS_LISTVALUES (V_LOGSTID, V_RQREQUIREMENTID);
		END IF;
	END IF;
END;
/


/****** Object:  StoredProcedure [RQDELETE_LOG_REQUIREMENT] ******/
CREATE OR REPLACE PROCEDURE RQDELETE_LOG_REQUIREMENT
(
	V_PROJECTGUID      VARCHAR2,
	V_REQUIREMENTGUID  VARCHAR2,
	V_VERSIONUSERID    NUMBER
)
IS
	--Local variable declarations
	V_RQPROJECTFLAGS NUMBER(10);
	V_LOGSTID NUMBER(10);
	V_ISDOC NUMBER(1);
	V_RQPROJECTMAPID VARCHAR2(40);
	V_RQUSERID VARCHAR2(20);
	V_EXTERNALID VARCHAR2(40);

BEGIN	
	SELECT FLAGS INTO V_RQPROJECTFLAGS FROM RQPROJECTS WHERE RQGUID = V_PROJECTGUID;
		
	IF (bitand(V_RQPROJECTFLAGS,64) = 0) THEN
		-- Sync for this specific project is OFF so do nothing here
		RETURN;
	END IF;
	
	BEGIN
		SELECT EXTERNALID INTO V_EXTERNALID FROM RQTLOGMAP WHERE OBJECTGUID=V_REQUIREMENTGUID AND OBJECTTYPE='RQREQUIREMENTS' AND EXTERNALID IS NOT NULL;
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			-- There is no record in RqTLogMap table so the requirement is not managed
			-- There is nothing else to do
			RETURN;
	END;
	
	-- get user guid and login name from id
	SELECT nvl(LOGINNAME,'UNKNOWN') INTO V_RQUSERID FROM RQUSERS WHERE ID=V_VERSIONUSERID;
	
	BEGIN
		-- get external map ids from RQTLOGMAP table
		SELECT EXTERNALID INTO V_RQPROJECTMAPID FROM RQTLOGMAP WHERE OBJECTGUID=V_PROJECTGUID AND OBJECTTYPE='RQPROJECTS';
	EXCEPTION
		WHEN NO_DATA_FOUND THEN
			V_RQPROJECTMAPID := NULL;
	END;
	
	-- insert column values into RQTLOG table
	RQINSERT_RQTLOG (V_PROJECTGUID, V_RQPROJECTMAPID, V_RQUSERID, SYSDATE, 0,
		3, 6, V_REQUIREMENTGUID, V_EXTERNALID, V_LOGSTID);
	
	-- Delete the requirement sync information for this requirement
	RQDELETE_RQTLOGMAP_ENTRY ('RQREQUIREMENTS', V_REQUIREMENTGUID);
	
END;
/


-- Since Oracle does not have any BITOR or BITXOR functions, we need to define them.
CREATE OR REPLACE FUNCTION bitor( x IN NUMBER, y IN NUMBER ) RETURN NUMBER  AS
BEGIN
    RETURN x + y - bitand(x,y);
END;
/


CREATE OR REPLACE FUNCTION bitxor( x IN NUMBER, y IN NUMBER ) RETURN NUMBER  AS
BEGIN
    RETURN bitor(x,y) - bitand(x,y);
END;
/


/*****************************************************************
*** When the composer project is deleted, we need to clear out ***
*** the ReqPro tlog tables to make the project available for   ***
*** connecting to another composer project.                    ***
******************************************************************/
CREATE OR REPLACE TRIGGER RQDELETE_PROJECT_TLOGMAP
   AFTER DELETE ON RQTLOGMAP
   FOR EACH ROW
   WHEN (old.objecttype = 'RQPROJECTS')
DECLARE
   pragma autonomous_transaction;
   v_projguid varchar2(40);
BEGIN
   v_projguid := :old.objectguid;
   
   delete from rqtlogproperty where projectguid = v_projguid;
   
   -- Deal with Requirement Types
   delete from rqtlogmap where id in (
      select m.id
      from rqtlogmap m, rqprojects p, rqprojectrequirementtypes pt, rqrequirementtypes t
      where p.rqguid=v_projguid and
         p.id=pt.projectid and
         pt.requirementtypeid= t.id and
         t.rqguid=m.objectguid);
   
   -- Deal with User Defined Fields
   delete from rqtlogmap where id in (
      select m.id
      from rqtlogmap m, rqprojects p, rqprojectrequirementtypes pt, rqrequirementtypes t, rqrequirementtypefields tf, rquserdefinedfields f
      where p.rqguid=v_projguid and
         p.id=pt.projectid and
         pt.requirementtypeid= t.id and
         t.id=tf.requirementtypeid and
         tf.userdefinedfieldid=f.id and
         f.rqguid=m.objectguid);
   
   -- Deal with User Defined List Items
   delete from rqtlogmap where id in (
      select m.id
      from rqtlogmap m, rqprojects p, rqprojectrequirementtypes pt, rqrequirementtypes t, rqrequirementtypefields tf, rquserdefinedfields f, rquserdefinedlistitems i
      where p.rqguid=v_projguid and
         p.id=pt.projectid and
         pt.requirementtypeid= t.id and
         t.id=tf.requirementtypeid and
         tf.userdefinedfieldid=f.id and
         f.id=i.fieldid and
         i.rqguid=m.objectguid);
   
   -- Deal with Requirements
   delete from rqtlogmap where id in (
      select m.id
      from rqtlogmap m, rqprojects p, rqprojectrequirements pr, rqrequirements r
      where p.rqguid=v_projguid and
         p.id=pr.projectid and
         pr.requirementid= r.id and
         r.rqguid=m.objectguid);
   
   -- Remove RRC link for this project's requirements
   delete from rqresources where id in (
      select rs.id
      from rqrequirements r, rqprojectrequirements pr, rqprojects p, rqresources rs
      where r.id = pr.requirementid and
         p.id = pr.projectid and
         r.id=rs.requirementid and
         rs.flags=2 and p.rqguid=v_projguid);

   
   --  Turn sync flag OFF, if it is ON, for this project
   update rqprojects set flags = bitxor(flags,64) 
   where (bitand(flags,64) <> 0) and
      rqguid = v_projguid;
   
   commit;
END;
/


/****** Object:  Trigger [RQREQUIREMENTHISTORY_ITRIG_LOG] ******/
CREATE OR REPLACE TRIGGER RQREQUIREMENTHISTORY_ITRIG_LOG
	BEFORE INSERT ON RQREQUIREMENTHISTORY
	FOR EACH ROW
DECLARE
	--Declarations of RQRequirement columns values
	V_RQREQGUID VARCHAR2(40);
	
	--Local variable declarations
	V_RQPROJECTGUID VARCHAR2(40);
	V_VERSIONUSERNAME VARCHAR2(40);
	V_RQPROJECTFLAGS NUMBER(10);
	V_RRMSYNCUSERNAME VARCHAR2(40);
	row_cnt NUMBER;
BEGIN
	-- Do not start work unless this is an update to an existing requirement.
	IF (:new.VERSIONNUMBER <> '1.0000') THEN
		-- Get login name of the user who updated the RequisitePro requirement.
		SELECT LOGINNAME INTO V_VERSIONUSERNAME FROM RqUsers WHERE id=:new.VERSIONUSERID;
		
		-- get project guid and flags from RQPROJECTS, userid of the updater, userid from RqUsers
		-- All from a single query to take advantage of the schema relationship and save calls and time.
		SELECT
			RQPROJECTS.FLAGS,
		 	RQPROJECTS.RQGUID,
		 	RQREQUIREMENTS.RQGUID,
		 	RQTLOGPROPERTY.VALUE
		INTO
			V_RQPROJECTFLAGS,
			V_RQPROJECTGUID,
			V_RQREQGUID,
			V_RRMSYNCUSERNAME
		FROM
			RQPROJECTS INNER JOIN
			RQPROJECTREQUIREMENTS ON RQPROJECTS.ID = RQPROJECTREQUIREMENTS.PROJECTID INNER JOIN
			RQTLOGPROPERTY ON RQPROJECTS.RQGUID = RQTLOGPROPERTY.PROJECTGUID INNER JOIN
			RQREQUIREMENTS ON RQPROJECTREQUIREMENTS.REQUIREMENTID = RQREQUIREMENTS.ID
		WHERE
			RQTLOGPROPERTY.KEYNAME = 'RRMSyncUser' AND
			RQPROJECTREQUIREMENTS.REQUIREMENTID = :new.ID;
		
		IF ( (bitand(V_RQPROJECTFLAGS,64) <> 0) AND (V_RRMSYNCUSERNAME <> V_VERSIONUSERNAME) ) THEN
			SELECT COUNT(*) INTO row_cnt FROM RQTLOGMAP WHERE OBJECTGUID=V_RQREQGUID AND OBJECTTYPE='RQREQUIREMENTS' AND EXTERNALID IS NOT NULL;
			IF (row_cnt > 0) THEN
			    -- insert entries into TLog tables for Requirement
			    RQINSERT_LOG_REQUIREMENT (V_RQPROJECTGUID, V_RQREQGUID, 2);
			END IF;
		END IF;
	END IF;
END;
/


-- Update flag for integration completion
UPDATE RQREQUISITE SET RESERVEDFLAGS = bitor(RESERVEDFLAGS, 2);

COMMIT;